from enum import IntEnum


class ResponseCodes(IntEnum):
    BAD_REQUEST_ERROR = 400
    AUTHENTICATION_ERROR = 401
    PERMISSION_DENIED_ERROR = 403
    UNPROCESSABLE_ENTITY_ERROR = 422
    NOT_FOUND_ERROR = 404
    RATE_LIMIT_ERROR = 429
    INTERNAL_SERVER_ERROR = 500
    CONFLICT_ERROR = 409
    CONTEXT_WINDOW_EXCEEDED_ERROR = 1001
    CONTENT_POLICY_VIOLATION_ERROR = 1002
    API_CONNECTION_ERROR = 1003
    API_TIMEOUT_ERROR = 1004
